function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function watch(target) {
  var subscribableTarget = target;
  var subscribers = [];

  function subscribe(cb, subscription) {
    subscribers.push({
      cb: cb,
      subscription: subscription
    });
    return function () {
      return unsubscribe(cb);
    };
  }

  function unsubscribe(removeCB) {
    var index = subscribers.findIndex(function (_ref) {
      var cb = _ref.cb;
      return cb === removeCB;
    });
    subscribers.splice(index, 1);
  }

  function notifyChangeFor(prop, obj) {
    subscribers.forEach(function (_ref2) {
      var subscription = _ref2.subscription,
          cb = _ref2.cb;

      if (!subscription || subscription[prop]) {
        cb(obj);
      }
    });
  }

  var p = new Proxy(subscribableTarget, {
    get: function get(obj, prop) {
      if (prop === 'subscribe') {
        return subscribe;
      }

      if (prop === 'unsubscribe') {
        return unsubscribe;
      }

      return obj[prop];
    },
    set: function set(obj, prop, value) {
      if (prop === 'subscribe' || prop === 'unsubscribe') {
        return true;
      }

      obj[prop] = value;
      notifyChangeFor(prop, _extends({}, obj));
      return true;
    }
  });
  return p;
}

function watchCollection(collection) {
  if (collection === void 0) {
    collection = {};
  }

  var subscribers = [];
  var collectionMethods = {
    subscribe: function subscribe(cb) {
      subscribers.push({
        cb: cb
      });
      return function () {
        return collectionMethods.unsubscribe(cb);
      };
    },
    unsubscribe: function unsubscribe(removeCB) {
      var index = subscribers.findIndex(function (_ref) {
        var cb = _ref.cb;
        return cb === removeCB;
      });
      subscribers.splice(index, 1);
    },
    add: function add(item) {
      return watchableCollection[item.id] = item;
    },
    remove: function remove(item) {
      delete watchableCollection[item.id];
    },
    find: function find(name) {
      return watchableCollection[name];
    },
    notifyChange: function notifyChange() {
      subscribers.forEach(function (_ref2) {
        var cb = _ref2.cb;
        cb();
      });
    }
  };
  var watchableCollection = new Proxy(collection, {
    get: function get(obj, prop) {
      var action = collectionMethods[prop];

      if (action) {
        return action;
      }

      return obj[prop];
    },
    set: function set(obj, prop, value) {
      if (prop === 'subscribe' || prop === 'unsubscribe') {
        return false;
      }

      obj[prop] = value;
      collectionMethods.notifyChange();
      return true;
    },
    deleteProperty: function deleteProperty(obj, prop) {
      if (prop === 'subscribe' || prop === 'unsubscribe') {
        return false;
      }

      delete obj[prop];
      collectionMethods.notifyChange();
      return true;
    }
  });
  return watchableCollection;
}

export { watch, watchCollection };
//# sourceMappingURL=babas.esm.js.map
